% This program computes the optimal policy and industry equilibrium for
% a given set of parameter values

% Main parameters
r=0.0425;
rho=0.0525;
alpha=0.0075; % This corresponds to my in the paper!
sigma=0.15;
lambda=0.04;

cost=0.12; % fixed cost per period (corresponds to 'c' in the paper)
phi=0.4865; % entry cost


% Units for money are in millions of dollars

% Transaction cost parameters for new cash injections
% These are from Hennessy and Whited (2007) (all firms)
ksi=0.0598; %fixed recapitalization cost
gamma=1.091; %marginal recapitalization cost NOTE! THis is different from the notation of the paper. This corresponds to 1+gamma of the paper!

% Corresponding parameters from Gomes (2001)
%ksi=0.08;
%gamma=1.028;

% Iteration parameters for optimal policy:
iternum=1000; % maximum number of iterations within a given grid
dvmax=0.001; % stops iteration if continuation region remains constant for "stopnum" successive iterations 
iterfine=3; % the number of times the grid is made finer
%iterfine=5; % To get more accurate policy, use this value
fineratio=2; % Divide dx and ds by this number to get a finer grid
divlumpratio=10^6; % Refers to the size of dividend payment

% Iteration parameters for industry equilibrium density:
iternumdensity=10000; % maximum number of iterations
density_tol=1e-10; % stopping criterium for difference between two successive iterations
zinit=1; % Initial productivity (normalization)


params=[alpha,sigma,r,rho,lambda,cost,ksi,gamma];

% This program uses an evenly spaced grid that includes x=0, and uses transition
% probability p so that variable 'x' is a arithmetic BM with drift my=alpha-(sigma^2)/2. 
% Variable y=exp(x) then follows GBM(alpha,sigma). Variable pi referes to
% profit, that is pi=y-cost.
% Note! 'alpha' in this program corresponds to 'my' in the paper, and 'my'
% in this program corresponds to 'my-(sigma^2)/2' in the paper !!!
xmax=3;
smax=4;
beta=1/2-alpha/(sigma^2)-sqrt((alpha/(sigma^2)-1/2)^2+2*(rho+lambda)/(sigma^2));
ystar=(beta/(beta-1))*((rho+lambda)-alpha)*(cost/(rho+lambda)); %unconstrained exit threshold (note that in this program y~GBM(alpha,sigma))
xmin=log(ystar)-0.1;
dxinit=0.1;  % Initial grid spacing for x
%    dsinit=0.1;  % Initial grid spacing for s
dsinit=2*cost*0.1;  % Initial grid spacing for s

% This initialization uses unconstrained value function to start
[xgridinit,sgridinit,Vinit]=makegridsalternative(dxinit,dsinit,xmin,xmax,smax,alpha,sigma,rho,lambda,cost);

%Compute optimal policy:
[params,xgrid,sgrid,V,pol]=liquidrun(r,rho,lambda,alpha,sigma,xgridinit,sgridinit,Vinit,divlumpratio,iternum,dvmax,iterfine,fineratio,[],[],[],ksi,gamma,cost);
dt=((xgrid(2)-xgrid(1))/sigma)^2;



% Draw optimal policy:
ygrid=exp(xgrid); % 'x' in this program refers to log(revenue), and 'y' refers to revenue (i.e. corresponds to 'x' in the paper)
figure;pcolor(ygrid,sgrid,pol);xlabel('revenue');ylabel('cash holdings');



entrycost=phi;

% Create transition matrix for iterating firm density:
[Tl,Td,polT,xgridT,sgridT,VT,BirthVector,price]=MakeTransAllowTail(params,pol,xgrid,sgrid,V,entrycost,zinit);

% Compute industry equilibrium:
if price>0,
    [density zmean smean turnover issuerate]=FindDensity(params,xgridT,sgridT,polT,Tl,Td,BirthVector,price,iternumdensity,density_tol);
else
    density=zeros(length(sgrid),length(xgrid));
    zmean=0;
    smean=0;
    turnover=0;
    issuerate=0;
end

zgrid=exp(xgrid)./price; % This is the grid for productivity
% Compute industry equilibrium without liquidity constraints:
[price_unc zmean_unc turnover_unc density_unc zgrid_unc]=ComputeUnc(params,entrycost,zinit);


% Report key figures:
% Turnover with liquidity constraint
turnover

% Turnover without liquidity constraint:
turnover_unc

% Recapitalization rate with liquidity constraints:
issuerate

% Percentage increase in price due to liquidity constraint:
priceincrease=100*(price-price_unc)/price_unc

% Percentate change in average productivity due to liquidity constraint:
outputchange=100*(zmean-zmean_unc)/zmean_unc
